
/* $Id: util.c,v 1.8 1998/03/13 18:52:13 kohl Exp $ */


/*
 *                      CUMULVS version 1.0:
 *      Computational Steering and Interactive Visualization
 *                   in Distributed Applications
 *           Oak Ridge National Laboratory, Oak Ridge TN.
 *      Authors:  James Arthur Kohl and Philip M. Papadopoulos
 *                   (C) 1996 All Rights Reserved
 *
 *                              NOTICE
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted
 * provided that the above copyright notice appear in all copies and
 * that both the copyright notice and this permission notice appear
 * in supporting documentation.
 *
 * Neither the Institution, Oak Ridge National Laboratory, nor the
 * Authors make any representations about the suitability of this
 * software for any purpose.  This software is provided ``as is''
 * without express or implied warranty.
 *
 * CUMULVS was funded by the U.S. Department of Energy.
 */


#include "slicer.h"
#include "slicer_glob.h"


void
safe_eval( itp, cmd )
Tcl_Interp *itp;
char *cmd;
{
	viewer_lock();

	Tcl_Eval( itp, cmd );

	viewer_unlock();
}


WIN create_win()
{
	WIN tmp;

	tmp = (WIN) MALLOC( sizeof( struct win_struct ) );
	memcheck( tmp, "Window Display Object" );

	tmp->canvas = (char *) NULL;

	tmp->htvar = (char *) NULL;
	tmp->wtvar = (char *) NULL;

	tmp->scale = -1.0;

	tmp->csx = tmp->csy = -1;

	stv_init_region( tmp->region );

	tmp->xoff = tmp->yoff = -1;

	tmp->border_id = -1;

	return( tmp );
}


void
free_win( ptr )
WIN *ptr;
{
	WIN W;

	if ( ptr == NULL || *ptr == NULL )
		return;
	
	W = *ptr;

	W->canvas = (char *) NULL;

	W->htvar = (char *) NULL;
	W->wtvar = (char *) NULL;

	W->scale = -1.0;

	W->csx = W->csy = -1;

	stv_init_region( W->region );

	W->xoff = W->yoff = -1;

	W->border_id = -1;

	FREE( W );

	*ptr = (WIN) NULL;
}


REGION_STACK create_region_stack()
{
	REGION_STACK tmp;

	int i;

	tmp = (REGION_STACK) MALLOC( sizeof( struct region_stack_struct ) );
	memcheck( tmp, "Region Stack Entry" );

	for ( i=0 ; i < STV_REGION_SIZE( STV_MAX_DIM ) ; i++ )
		tmp->region[i] = -1;
	
	tmp->next = (REGION_STACK) NULL;

	return( tmp );
}


void
free_region_stack( ptr )
REGION_STACK *ptr;
{
	REGION_STACK RS;

	int i;

	if ( ptr == NULL || *ptr == NULL )
		return;
	
	RS = *ptr;

	for ( i=0 ; i < STV_REGION_SIZE( STV_MAX_DIM ) ; i++ )
		RS->region[i] = -1;
	
	RS->next = (REGION_STACK) NULL;

	FREE( RS );

	*ptr = (REGION_STACK) NULL;
}


void
push_region( R )
STV_REGION R;
{
	REGION_STACK RS;

	RS = create_region_stack();

	stv_copy_region( RS->region, R );

	RS->next = REGIONS;

	REGIONS = RS;
}


int
pop_region( R )
STV_REGION R;
{
	REGION_STACK RS;

	if ( REGIONS != NULL )
	{
		stv_copy_region( R, REGIONS->region );

		RS = REGIONS->next;

		free_region_stack( &REGIONS );

		REGIONS = RS;

		return( stvTrue );
	}

	return( stvFalse );
}


int
thrashing()
{
	static struct timeval last = { -1, -1 };
	static int phase = 0;
	static int count = 0;

	struct timeval tmp;

	float diff;

	/* Reset */

	if ( THRASH_FACTOR < 0 )
	{
		THRASH_FACTOR = 1;

		phase = 0;
	}

	/* Check Timing */

	if ( !phase )
	{
		gettimeofday( &tmp, (struct timezone *) NULL );

		diff = ( (float) ( tmp.tv_sec - last.tv_sec ) * 1000000.0 )
			+ (float) ( tmp.tv_usec - last.tv_usec );

		last.tv_sec = tmp.tv_sec;
		last.tv_usec = tmp.tv_usec;

		if ( diff < 500000 )
		{
			/* Increase Backoff */

			if ( count < 0 )
			{
				THRASH_FACTOR *= 2;

				phase = THRASH_FACTOR;
			}

			/* Kick-In Threshold */

			else
			{
				count++;

				if ( count > 3 )
					phase = THRASH_FACTOR;
			}

			return( stvTrue );
		}
		
		else
		{
			THRASH_FACTOR = 1;

			count = 0;

			return( stvFalse );
		}
	}

	/* Exponential Backoff & Retry */

	else
	{
		count = -1;

		phase--;

		if ( !phase )
			gettimeofday( &last, (struct timezone *) NULL );

		return( stvTrue );
	}
}


char *
center_str( str, width )
char *str;
int width;
{
	char *tmp;

	int i, j;
	int len;

	tmp = (char *) MALLOC( (STV_SIZE_T) (width + 1) * sizeof(char) );
	memcheck( tmp, "Center String" );

	len = strlen( str );

	if ( len > width )
	{
		strcpy( tmp, (char *) ( str + ( len - width ) ) );

		tmp[0] = '<';
	}

	else
	{
		j = ( width - len ) / 2;

		for ( i=0 ; i < j ; i++ )
			tmp[i] = ' ';

		strcpy( (char *) ( tmp + j ), str );

		for ( i=(j + len) ; i < width ; i++ )
			tmp[i] = ' ';

		tmp[width] = '\0';
	}

	return( tmp );
}


char *
pad_num( num, max )
long num;
long max;
{
	char tmp[1024];

	char *ptr;
	char *str;

	int nd;
	int nz;
	int i;

	sprintf( tmp, "%ld", num );

	nd = strlen( tmp );

	nz = max - nd;

	str = (char *) MALLOC( (STV_SIZE_T) (max + 1) * sizeof(char) );
	memcheck( str, "Numerical Pad String" );

	ptr = str;

	for ( i=0 ; i < nz ; i++ )
		*ptr++ = '0';

	sprintf( (char *) (str + nz), "%ld", num );

	return( str );
}


char *
trunc_str( str, len, mark_incl )
char *str;
int len;
int mark_incl;
{
	char tmp[1024];

	char *ptr;

	int slen;

	sprintf( tmp, "%s", str );

	slen = strlen( str );

	if ( slen > len )
	{
		if ( mark_incl )
			ptr = tmp + len - 1;
		
		else
			ptr = tmp + len;

		*ptr++ = '*';

		*ptr = '\0';
	}

	return( stv_copy_str( tmp ) );
}


char *
upper_str( str )
char *str;
{
	char *ptr;
	char *tmp;

	tmp = stv_copy_str( str );

	ptr = tmp;

	while ( *ptr != '\0' )
	{
		if ( *ptr >= 'a' && *ptr <= 'z' )
			*ptr += 'A' - 'a';

		ptr++;
	}

	return( tmp );
}


int
append_cmd_str( result, str, length, err, cmd_flag )
char *result;
char *str;
int length;
char *err;
int cmd_flag;
{
	char *strcat();

	char *errstr;

	int elen;
	int rlen;
	int len;
	int rc;
	int i;

	rlen = strlen( result );

	len = rlen + strlen( str ) + 1;

	if ( len <= length )
		strcat( result, str );

	else
	{
		if ( cmd_flag == stvTrue )
		{
			if ( err == NULL )
			{
				printf( "\nError in Append Cmd String: Null Cmd\n\n" );

				return( TCL_ERROR );
			}

			elen = len + strlen( err ) + 5;

			errstr = (char *) MALLOC( (STV_SIZE_T) elen
				* sizeof(char) );
			memcheck( errstr, "Append Cmd String Err String" );

			sprintf( errstr, "%s { %s }", err, result );

			rc = Tcl_Eval( interp, errstr );

			if ( rc == TCL_ERROR )
				printf( "\nError Executing \"%s\"\n\n", errstr );

			FREE( errstr );

			if ( strlen( str ) <= length )
				strcpy( result, str );

			else
			{
				printf( "\nError Append Cmd String \"%s\" Too Long\n\n",
					str );

				strcpy( result, "" );
			}

			return( rc );
		}

		else
		{
			if ( err != NULL )
				printf( "\nError in Append Cmd String: %s\n\n", err );

			i = rlen;

			while ( i < length - 4 )
			{
				result[i] = str[ i - rlen ];

				i++;
			}

			sprintf( (char *) (result + (length - 4)), "..." );
		}
	}

	return( TCL_OK );
}


double
distance( x1, y1, x2, y2 )
double x1, y1, x2, y2;
{
	return( sqrt(((x2 - x1) * (x2 - x1)) + ((y2 - y1) * (y2 - y1))) );
}


double
angle_of( x1, y1, x2, y2 )
double x1, y1, x2, y2;
{
	double slope;
	double at;

	/* North is heading angle 0.0 */
	/* y's grow downward from the top of the screen */

	if ( x2 == x1 )
	{
		if ( y2 > y1 )
			at = -1.0 * PI;

		else
			at = 0.0;
	}

	else
	{
		slope = (y1 - y2) / (x2 - x1);

		at = atan( slope );

		if ( x1 > x2 )
			at += PI / 2.0;

		else
			at -= PI / 2.0;
	}

	return( at );
}


int
filecheck( fp, name )
FILE *fp;
char *name;
{
	char msg[1024];
	char tmp[1024];

	if ( fp == NULL )
	{
		sprintf( tmp, "Error Opening File \"%s\"", name );

		sprintf( msg, "setMsg {%s}", tmp );

		Tcl_Eval( interp, msg );

		fprintf( stderr, "\n%s\n\n", tmp );

		return( stvFalse );
	}

	return( stvTrue );
}


void
memcheck( ptr, name )
char *ptr;
char *name;
{
	if ( ptr == NULL )
	{
		fprintf( stderr, "\nError Allocating Memory for \"%s\"\n\n",
			name );

		exit( -1 );
	}
}


int
compare( x, y )
char *x, *y;
{
	while ( *x != '\0' )
	{
		if ( *x != *y )
			return( stvFalse );

		x++; y++;
	}

	return( stvTrue );
}

